@@ -32,10 +32,12 @@
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <sys/param.h>
+#include <sys/stat.h>
 
 #include <linux/if_addr.h>
 #include <linux/if_link.h>
 #include <linux/if_packet.h>
+#include <linux/if_tun.h>
 #include <linux/if_vlan.h>
 #include <linux/filter.h>
 #include <linux/netlink.h>
@@ -98,11 +100,6 @@ int if_getssid_wext(const char *ifname, uint8_t *ssid);
 #define	SOL_NETLINK	270
 #endif
 
-#define bpf_insn		sock_filter
-#define BPF_SKIPTYPE
-#define BPF_ETHCOOK		-ETH_HLEN
-#define BPF_WHOLEPACKET	0x0fffffff /* work around buggy LPF filters */
-
 struct priv {
 	int route_fd;
 	uint32_t route_pid;
@@ -122,7 +119,9 @@ static const uint8_t ipv4_bcast_addr[] = {
 
 #define PROC_INET6	"/proc/net/if_inet6"
 #define PROC_PROMOTE	"/proc/sys/net/ipv4/conf/%s/promote_secondaries"
+#define SYS_BRIDGE	"/sys/class/net/%s/bridge"
 #define SYS_LAYER2	"/sys/class/net/%s/device/layer2"
+#define SYS_TUNTAP	"/sys/class/net/%s/tun_flags"
 
 static const char *mproc =
 #if defined(__alpha__)
@@ -205,6 +204,20 @@ check_proc_int(const char *path)
 	return i;
 }
 
+static int
+check_proc_hex(const char *path, unsigned int *value)
+{
+	FILE *fp;
+	int i;
+
+	fp = fopen(path, "r");
+	if (fp == NULL)
+		return -1;
+	i = fscanf(fp, "%x", value) == 1 ? 0 : -1;
+	fclose(fp);
+	return i;
+}
+
 static ssize_t
 write_path(const char *path, const char *val)
 {
@@ -257,6 +270,39 @@ if_conf(struct interface *ifp)
 	return 0;
 }
 
+static bool
+if_bridge(const char *ifname)
+{
+	char path[sizeof(SYS_BRIDGE) + IF_NAMESIZE];
+	struct stat sb;
+
+	snprintf(path, sizeof(path), SYS_BRIDGE, ifname);
+	if (stat(path, &sb) == 0 && S_ISDIR(sb.st_mode))
+		return true;
+	return false;
+}
+
+static bool
+if_tap(const char *ifname)
+{
+	char path[sizeof(SYS_TUNTAP) + IF_NAMESIZE];
+	unsigned int n;
+
+	snprintf(path, sizeof(path), SYS_TUNTAP, ifname);
+	if (check_proc_hex(path, &n) == -1)
+		return false;
+	return n & IFF_TAP;
+}
+
+bool
+if_ignore(__unused struct dhcpcd_ctx *ctx, const char *ifname)
+{
+
+	if (if_tap(ifname) || if_bridge(ifname))
+		return true;
+	return false;
+}
+
 /* XXX work out Virtal Interface Masters */
 int
 if_vimaster(__unused const struct dhcpcd_ctx *ctx, __unused const char *ifname)
@@ -1337,45 +1383,55 @@ if_initrt(struct dhcpcd_ctx *ctx, rb_tree_t *kroutes, int af)
 #ifdef INET
 /* Linux is a special snowflake when it comes to BPF. */
 const char *bpf_name = "Packet Socket";
-#define	BPF_BUFFER_LEN		1500
 
+/* Linux is a special snowflake for opening BPF. */
 int
 bpf_open(struct interface *ifp, int (*filter)(struct interface *, int))
 {
 	struct ipv4_state *state;
-/* Linux is a special snowflake for opening BPF. */
 	int s;
 	union sockunion {
 		struct sockaddr sa;
 		struct sockaddr_ll sll;
 		struct sockaddr_storage ss;
-	} su;
+	} su = {
+		.sll.sll_family = PF_PACKET,
+		.sll.sll_protocol = htons(ETH_P_ALL),
+		.sll.sll_ifindex = (int)ifp->index,
+	};
 #ifdef PACKET_AUXDATA
 	int n;
 #endif
 
+	/* Allocate a suitably large buffer for a single packet. */
+	state = ipv4_getstate(ifp);
+	if (state == NULL)
+		return -1;
+	if (state->buffer_size < ETH_DATA_LEN) {
+		void *nb;
+
+		if ((nb = realloc(state->buffer, ETH_DATA_LEN)) == NULL)
+			return -1;
+		state->buffer = nb;
+		state->buffer_size = ETH_DATA_LEN;
+		state->buffer_len = state->buffer_pos = 0;
+	}
+
+
 #define SF	SOCK_CLOEXEC | SOCK_NONBLOCK
 	if ((s = xsocket(PF_PACKET, SOCK_RAW | SF, htons(ETH_P_ALL))) == -1)
 		return -1;
 #undef SF
 
-	/* Allocate a suitably large buffer for a single packet. */
-	state = ipv4_getstate(ifp);
-	if (state == NULL)
+	/* We cannot validate the correct interface,
+	 * so we MUST set this first. */
+	if (bind(s, &su.sa, sizeof(su.sll)) == -1)
 		goto eexit;
-	if (state->buffer_size < ETH_DATA_LEN) {
-		void *nb;
-
-		if ((nb = realloc(state->buffer, ETH_DATA_LEN)) == NULL)
-			goto eexit;
-		state->buffer = nb;
-		state->buffer_size = ETH_DATA_LEN;
-		state->buffer_len = state->buffer_pos = 0;
-	}
 
 	if (filter(ifp, s) != 0)
 		goto eexit;
 
+	/* In the ideal world, this would be set before the bind and filter. */
 #ifdef PACKET_AUXDATA
 	n = 1;
 	if (setsockopt(s, SOL_PACKET, PACKET_AUXDATA, &n, sizeof(n)) != 0) {
@@ -1384,19 +1440,18 @@ bpf_open(struct interface *ifp, int (*filter)(struct interface *, int))
 	}
 #endif
 
-	memset(&su, 0, sizeof(su));
-	su.sll.sll_family = PF_PACKET;
-	su.sll.sll_protocol = htons(ETH_P_ALL);
-	su.sll.sll_ifindex = (int)ifp->index;
-	if (bind(s, &su.sa, sizeof(su.sll)) == -1)
-		goto eexit;
+	/*
+	 * At this point we could have received packets for the wrong
+	 * interface or which don't pass the filter.
+	 * Linux should flush upon setting the filter like every other OS.
+	 * There is no way of flushing them from userland.
+	 * As such, consumers need to inspect each packet to ensure it's valid.
+	 * Or to put it another way, don't trust the Linux BPF filter.
+	*/
+
 	return s;
 
 eexit:
-	if (state != NULL) {
-		free(state->buffer);
-		state->buffer = NULL;
-	}
 	close(s);
 	return -1;
 }
